"use client";
import { useRef, useState } from "react";
import {
  Box,
  Button,
  Checkbox,
  Flex,
  FormControl,
  Table,
  Tbody,
  Td,
  Th,
  Thead,
  Tr,
} from "@chakra-ui/react";
import * as Yup from "yup";
import FormComponent from "@/components/form/Form";
import { createRole, updateRole } from "@/app/lib/role/action";
import useShowToast from "@/app/hooks/useShowToast";
import { useParams, useRouter } from "next/navigation";
import { InputWrapper } from "@/components/fields/Fields";
import { ProjectInterface, ProjectOptionInterface } from "@/types/project";
import MultiSelectWrapper from "@/components/fields/MultiSelectWrapper";

type Permissions = string[];

interface RoleFormProps {
  modules: { [key: string]: string[] };
  initialPermissions?: Permissions;
  currentProjects?: ProjectOptionInterface[];
  roleName?: string;
  isEdit?: boolean;
  projects: ProjectInterface[];
}

interface FormData {
  Permissions: Permissions;
  Projects: number[];
  RoleName?: string;
}

const validationSchema = Yup.object({
  RoleName: Yup.string().required("Role name is required"),
});

export default function RoleForm({
  modules,
  initialPermissions,
  roleName,
  projects,
  currentProjects,
  isEdit,
}: RoleFormProps) {
  const router = useRouter();
  const formRef = useRef(null);
  const showToast = useShowToast();
  const params = useParams<{ id: string }>();
  const [loading, setLoading] = useState(false);
  const [permissions, setPermissions] = useState<Permissions>(
    initialPermissions || []
  );
  const [projectVal, setProjectVal] = useState<ProjectOptionInterface[]>(currentProjects || []);

  const initialValues = {
    RoleName: roleName || "",
    Projects: projectVal,
    Permissions: initialPermissions, 
  };

  const handleCreate = async (values: FormData) => {
    setLoading(true);
    try {
      const response = await createRole({
        name: values.RoleName,
        projects: values.Projects,
        permissions: permissions
      });
      showToast("Role Created!", response.message, "success");
      router.push("/admin/roles");
    } catch (error: any) {
      showToast("Failed to create", error.message, "error");
    } finally {
      setLoading(false);
    }
  };

  const handleUpdate = async (values: FormData) => {
    setLoading(true);
    try {
      const response = await updateRole({
        id: Number(params.id),
        name: values.RoleName,
        projects: values.Projects,
        permissions: permissions,
      });
      showToast("Role Updated!", response.message, "success");
      router.push("/admin/roles");
    } catch (error: any) {
      showToast("Failed to update", error.message, "error");
    } finally {
      setLoading(false);
    }
  };

  const handleSubmit = (values: FormData) => {
    if (isEdit) {
      handleUpdate(values);
    } else {
      handleCreate(values);
    }
  };

  const handleSubmitClick = () => {
    formRef?.current?.externalHandleSubmit();
  };

  const handlePermissionChange = (permission: string) => {
    setPermissions((prevPermissions) =>
      prevPermissions.includes(permission)
        ? prevPermissions.filter((perm) => perm !== permission)
        : [...prevPermissions, permission]
    );
  };

  return (
    <Box pt={{ base: "130px", md: "80px", xl: "80px" }} mb="20px">
      <FormComponent ref={formRef} submitForm={handleSubmit} initialValues={initialValues} validationSchema={validationSchema}>
        <Box display="flex" gap={15}>
          <FormControl w="50%">
            <InputWrapper id="name" type="text" fieldName="RoleName" label="Role Name" placeholder="Enter Role Name" extra="*" defaultValue={initialValues.RoleName} />
          </FormControl>
          <FormControl w="50%">
            <MultiSelectWrapper
              id="projects"
              fieldName="Projects"
              label="Projects"
              options={projects?.map((item: any) => {
                return { value: item?.id, label: item?.name };
              })}
              defaultValue={
                initialValues.Projects || []
              }
              placeholder="Select Projects"
              extra="*"
            />
          </FormControl>
        </Box>

        {/* Permissions Table */}
        <Table variant="simple">
          <Thead>
            <Tr>
              <Th>Module</Th>
              <Th>View</Th>
              <Th>Create</Th>
              <Th>Edit</Th>
              <Th>Delete</Th>
            </Tr>
          </Thead>
          <Tbody>
            {Object.entries(modules).map(([moduleName]) => (
              <Tr key={moduleName}>
                <Td>{moduleName}</Td>
                {modules[moduleName].map((permission) => (
                  <Td key={permission}>
                    <Checkbox isChecked={permissions.includes(permission)} onChange={() => handlePermissionChange(permission)} />
                  </Td>
                ))}
              </Tr>
            ))}
          </Tbody>
        </Table>
        <Flex w={"full"} justifyContent={"flex-end"} pt={4}>
          <Button variant="brand" onClick={handleSubmitClick} mr={3} w={"80px"} justifySelf={"flex-end"} isDisabled={loading}>
            {isEdit ? "Save" : "Create"}
          </Button>
        </Flex>
      </FormComponent>
    </Box>
  );
};